// MyTronBot.cc
//
// This is the code file that you will modify in order to create your entry.

#include "time_util.h"
#include "Map.h"
#include "alphabeta.h"
#include "territory.h"
#include "connectivity.h"
#include "endgame.h"
#include "SFMT.h"
#include <string>
#include <vector>

int makeMove(Map& map, ABSearch& search, EndSearch& endgame)
{
    Territory* p1t = Territory::allocate();
    Territory* p2t = Territory::allocate();
    bool have_shared = map.space->separate(map.bikes[0], map.bikes[1],
            *p1t, *p2t);
    Territory::free(p1t);
    Territory::free(p2t);
    if (have_shared)
    {
        return search.search(map);
    }
    else
    {
        return endgame.search(map);
    }
}

int main() {
    init_gen_rand(0xF00F);
    init_timer();
    StopWatch clock;
    clock.set_length(950);
    NodeTable ttable = NodeTable();
    cnct::TreeSizeCache tscache = cnct::TreeSizeCache();
    EndSearch endgame = EndSearch(&clock, &ttable, &tscache);
    ABSearch search = ABSearch(&clock, &ttable, &tscache, &endgame);
#ifdef MEMORY_CHECK
    cnct::Node::allocated = 0;
    cnct::Tree::allocated = 0;
    Territory::allocated = 0;
#endif
    while (true)
    {
        Map map;
        clock.start();
#ifdef TREE_REPORT
        std::string astr = map.toString();
        fprintf(stderr, "Map:\n%s", astr.c_str());
        cnct::Tree tree = cnct::Tree(*map.space, map.bikes[0]);
        astr = tree.toString();
        fprintf(stderr, "Chambers:\n%s", astr.c_str());
        fprintf(stderr, "Tree size: %d\n", tree.root->my_size);
#endif
        Map::makeMove(makeMove(map, search, endgame));
#ifdef MEMORY_CHECK
        if (cnct::Node::unreturned() != 0)
        {
            fprintf(stderr, "Leaked cnodes %d", cnct::Node::unreturned());
        }
        if (cnct::Tree::unreturned() != 0)
        {
            fprintf(stderr, "Leaked cTrees %d", cnct::Tree::unreturned());
        }
        if (Territory::unreturned() != 0)
        {
            fprintf(stderr, "Unreturned territories %d",
                    Territory::unreturned());
        }
#endif
    }
    return 0;
}
