
#define __STDC_CONSTANT_MACROS

#include "time_util.h"
#include "Map.h"
#include "endgame.h"
#include "connectivity.h"

#pragma once


enum ScoreLimits {
    TIE = 0,
    MIN_WIN = 25000,
    WIN = 50000,
    ABORT = 110000
};

class Node
{
    public:
    int age;
    uint64_t hash;
    int depth;
    int score;
    Bound bound;
    int move_order;

    Node();
};

struct NodeTableCell
{
    Node always;
    Node deepest;
};

class NodeTable
{
    public:
    NodeTable();
    const Node* get(const Map& position);
    void set(const Map& position, int ndepth, int nscore, Bound nbound,
            int norder);
    void ageTable();

    int overwrite;

    private:
    NodeTableCell* table;
    int table_size;
    int current_age;
};

class ABSearch
{
    public:
    ABSearch(StopWatch* sw, NodeTable* table, cnct::TreeSizeCache* tsc,
            EndSearch* es);
    int alphabeta(Map& position, int alpha, int beta, int depth);
    int search(Map& position);
    int eval(Map& position, Territory* p1territory, Territory* p2territory);

    private:
    StopWatch* clock;
    NodeTable* ttable;
    cnct::TreeSizeCache* tscache;
    EndSearch* endgame;

};

